// 
// Patron pour la generation d'une classe derivee dans ROOT
// FTTreeReader derive de FDataReader
// 
 
#ifndef FTTreeReader_h 
#define FTTreeReader_h 
 
#include "FDataReader.h" 
#include "TTree.h" 
#include "FParamTreeFormula.h" 
#include "TClass.h" 
 
class FTTreeReader : public FDataReader { 
 
   protected:
	TString fileName;
	TString treeName;
   	TTree *tree;
	Bool_t   addParamsFromBranch;
   	Long64_t maxEntries;
   	Long64_t curEntry;
	Long64_t nevt;

	Bool_t stop;

	Bool_t treeChanged;

   protected: 
        virtual void Init(void);       // initialisation 
	virtual void AddParamsFromBranch(TBranch *br, TClass *cl);
 
   public:	 
   	FTTreeReader(void);    // constructeur par defaut 
	FTTreeReader(const FTTreeReader &ap); // constructeur par copie 
	
	~FTTreeReader(void);		// desctructeur; 
	 
#if ROOT_VERSION_CODE >= ROOT_VERSION(3,4,0)
   	virtual void Copy(TObject & obj) const;
#else
   	virtual void Copy(TObject & obj);
#endif
	FTTreeReader& operator = (const FTTreeReader &hp); // operateur = 
 
	virtual void InitListOfParamsBruts(void);
	virtual void SetParamBrutsValues(void);
	virtual Bool_t IsOk(void){return !stop;}

	virtual void SetTree(void);
	virtual TTree *GetTree(void) const {return tree;}
	virtual TList *GetListOfCandidates(TClass *cl, const Char_t *prefix="");
	virtual TList *GetListOfAllCandidates(void);

        virtual void SetFileName(const Char_t *ser=0);
        virtual const Char_t *GetFileName(void){return fileName.Data();}
        virtual void SetTreeName(const Char_t *ser=0);
        virtual const Char_t *GetTreeName(void){return treeName.Data();}

        virtual Char_t const *GetDataReaderInfo(void);

 	virtual Bool_t IsTreeChanged(void) {return treeChanged;}
        
	virtual void InitFromEnv(TEnv *env);

        virtual void DoAddParamFromBranch(Bool_t d=kFALSE) {addParamsFromBranch=d;}
        virtual Bool_t IsDoingAddParamFromBranch(void) {return addParamsFromBranch;}

    ClassDef(FTTreeReader,1)
 
}; 
 
#endif 
 
