//
// Patron pour la generation d'une classe derivee dans ROOT
// FBlockDataReader derive de FDataReaderWithDevice
//

#ifndef FBlockDataReader_h
#define FBlockDataReader_h

#define MAXEVENTMULT 2048


#include "FDataReaderWithDevice.h"
#include "FParamBrut.h"

class FBlockDataReader : public FDataReaderWithDevice {

   protected:
        Bool_t swap;
        UChar_t *buf; //!
        Int_t bufSize;
        Int_t index;
        Int_t nbuf;
        Long_t evtNum;
        Short_t mult;
        Short_t label[MAXEVENTMULT];
        Short_t valeur[MAXEVENTMULT];

	Int_t nbOfParamBruts;
	FParamBrut **arrayOfParams; //!

        Long_t nevt;

	Bool_t stop;


   protected:
        virtual void Init(void);       // initialisation
	virtual void GetBuffer(void);
	virtual Int_t GetNextEvent(void);
        void GetNumber(void *num, Int_t size);
        void SwapNumber(void *num, Int_t size);
	virtual TString GetASCII(Int_t size);
	virtual Short_t GetNbOfParameters(void){return mult;}
	virtual Short_t GetLabel(Int_t i){return label[i];}
	virtual Short_t GetValue(Int_t i){return valeur[i];}

   public:
        enum FBlockReadStatus
	 {
	 readOk,
	 readError,
	 readStop
	 };

   public:
   	FBlockDataReader(void);    // constructeur par defaut
	FBlockDataReader(const FBlockDataReader &ap); // constructeur par copie

	~FBlockDataReader(void);		// desctructeur;

#if ROOT_VERSION_CODE >= ROOT_VERSION(3,4,0)
   	virtual void Copy(TObject & obj) const;
#else
   	virtual void Copy(TObject & obj);
#endif
	FBlockDataReader& operator = (const FBlockDataReader &hp); // operateur =

	virtual void InitListOfParamsBruts(void);
	virtual void SetParamBrutsValues(void);
	virtual Bool_t IsOk(void){return !stop;}

	virtual void SetSwap(Bool_t s=kFALSE){swap=s;}
	virtual Bool_t GetSwap(void){return swap;}
	virtual Long_t GetEventNumber(void){return evtNum;}
	virtual Long_t GetNbOfBuffers(void){return nbuf;}
	virtual FParamBrut *GetParamBrut(Int_t label);

	virtual void InitFromEnv(TEnv *env);

    ClassDef(FBlockDataReader,1)

};

#endif

