// 
// Patron pour la generation d'une classe derivee dans ROOT
// FGenericHistogramBuilder derive de TNamed
// 
 
#ifndef FGenericHistogramBuilder_h 
#define FGenericHistogramBuilder_h 
 
#include "TNamed.h" 
#include "TString.h" 
#include "FEventProcessor.h" 
 
class FGenericHistogramBuilder : public TNamed { 
 
   private: 
        TString		histName;
        TString		histTitle;
        TString		histClass;
        TString		paramX;
        TString		paramY;
        TString		condition;
	Int_t		nx;
	Double_t	xmin;
	Double_t	xmax;
	Int_t		ny;
	Double_t	ymin;
	Double_t	ymax;
	FEventProcessor *fep;
	
	
   protected: 
        virtual void Init(void);       // initialisation 
 
   public:	 
   	FGenericHistogramBuilder(void);    // constructeur par defaut 
	FGenericHistogramBuilder(const FGenericHistogramBuilder &ap); // constructeur par copie 
	
	~FGenericHistogramBuilder(void);		// desctructeur; 
	 
#if ROOT_VERSION_CODE >= ROOT_VERSION(3,4,0)
   	virtual void Copy(TObject & obj) const;
#else
   	virtual void Copy(TObject & obj);
#endif
	FGenericHistogramBuilder& operator = (const FGenericHistogramBuilder &hp); // operateur = 
 
        virtual void SetHistName(Char_t const *n=""){histName=n;}
        virtual void SetHistTitle(Char_t const *n=""){histTitle=n;}
        virtual void SetHistClass(Char_t const *n=""){histClass=n;}
        virtual void SetParamX(Char_t const *n=""){paramX=n;}
        virtual void SetParamY(Char_t const *n=""){paramY=n;}
        virtual void SetCondition(Char_t const *n=""){condition=n;}
	virtual void SetNbinsX(Int_t n=0){nx=n;}
	virtual void SetXmin(Double_t n=0.){xmin=n;}
	virtual void SetXmax(Double_t n=0.){xmax=n;}
	virtual void SetNbinsY(Int_t n=0){ny=n;}
	virtual void SetYmin(Double_t n=0.){ymin=n;}
	virtual void SetYmax(Double_t n=0.){ymax=n;}
	virtual void SetEventProcessor(FEventProcessor *f=0){fep=f;}
	
        virtual TString	 GetHistName(void){return histName;}
        virtual TString	 GetHistTitle(void){return histTitle;}
        virtual TString	 GetHistClass(void){return histClass;}
        virtual TString	 GetParamX(void){return paramX;}
        virtual TString	 GetParamY(void){return paramY;}
        virtual TString	 GetCondition(void){return condition;}
	virtual Int_t	 GetNbinsX(void){return nx;}
	virtual Double_t GetXmin(void){return xmin;}
	virtual Double_t GetXmax(void){return xmax;}
	virtual Int_t	 GetNbinsY(void){return ny;}
	virtual Double_t GetYmin(void){return ymin;}
	virtual Double_t GetYmax(void){return ymax;}
	virtual FEventProcessor *GetEventProcessor(void){return fep;}
	
	virtual void BuildHistograms(void);
	
    ClassDef(FGenericHistogramBuilder,1)
 
}; 
 
#endif 
 
