// 
// Patron pour la generation d'une classe derivee dans ROOT
// FIPDevice derive de FDevice
// 
 
#ifndef FIPDevice_h 
#define FIPDevice_h 
 
#define MAXREADBUFSIZE 65536

#include "FDevice.h" 
#include "FIPConnection.h" 
#include "TEnv.h" 
 
class FIPDevice : public FDevice { 

   protected:
	FIPConnection *sock;
	Bool_t tcp;

	TString server;
	Int_t port;
	TString protocol;
	Bool_t swap;

        UChar_t buf[MAXREADBUFSIZE];
 
   protected: 
        virtual void Init(void);       // initialisation 
 
   public:	 
   	FIPDevice(void);    // constructeur par defaut 
	FIPDevice(const FIPDevice &ap); // constructeur par copie 
	
	~FIPDevice(void);		// desctructeur; 
	 
#if ROOT_VERSION_CODE >= ROOT_VERSION(3,4,0)
   	virtual void Copy(TObject & obj) const;
#else
   	virtual void Copy(TObject & obj);
#endif
	FIPDevice& operator = (const FIPDevice &hp); // operateur = 
	
	virtual void Open(void);
	virtual void Close(void);
	virtual void Rewind(void);
	virtual UChar_t *GetBuffer(Int_t &size);
 
 	virtual void InitSocket(const Char_t *ser=0, 
	                        Int_t port=0, 
				const Char_t *prot=0);
        virtual void SetServer(const Char_t *ser=0);
        virtual void SetPort(Int_t port=0);
        virtual void SetProtocol(const Char_t *prot=0);
        virtual const Char_t *GetServer(void){return server.Data();}
        virtual Int_t GetPort(void){return port;}
        virtual const Char_t *GetProtocol(void){return protocol.Data();}
        virtual void WaitServerToStart(void);
        virtual void ReleaseSocket(void);
	virtual void SetSwap(Bool_t s=kTRUE) {swap=s;}
	virtual Bool_t GetSwap(void) {return swap;}
        virtual FIPConnection *GetSocket(void) const {return sock;}
	virtual void InitFromEnv(TEnv *env);

    ClassDef(FIPDevice,1)
 
}; 
 
#endif 
 
