// 
// Patron pour la generation d'une classe derivee dans ROOT
// FNetEventReader derive de TNamed
// 
 
#ifndef FNetEventReader_h 
#define FNetEventReader_h 
 
#include "TNamed.h" 
#include "TSocket.h" 
#include "FNetConnection.h" 

#define MAXREADBUFSIZE 32764
#define MAXEVENTMULT 2048
 
class FNetEventReader : public FNetConnection { 
 
   protected:
        UChar_t buf[MAXREADBUFSIZE];
        Bool_t swap;
        Int_t bufSize;
        Int_t index;
        Int_t nbuf;
        Long_t evtNum;
        Short_t mult;
        Short_t label[MAXEVENTMULT];
        Short_t valeur[MAXEVENTMULT];

   protected: 
        virtual void Init(void);       // initialisation 
	virtual void GetBuffer(void);
        void GetNumber(void *num, Int_t size);
        void SwapNumber(UChar_t *num, Int_t size);
 
   public:
        enum FNetEventReadStatus 
	 {
	 readOk,
	 readError
	 };

   public:	 

   	FNetEventReader(void);    // constructeur par defaut 
	FNetEventReader(const FNetEventReader &ap); // constructeur par copie 
	
	~FNetEventReader(void);		// desctructeur; 
	 
#if ROOT_VERSION_CODE >= ROOT_VERSION(3,4,0)
   	virtual void Copy(TObject & obj) const;
#else
   	virtual void Copy(TObject & obj);
#endif
	FNetEventReader& operator = (const FNetEventReader &hp); // operateur = 

 	virtual void InitSocket(const Char_t *ser=0, Int_t port=0);
	virtual void WaitServerToStart(void);
	
	virtual Int_t GetNextEvent(void);
	
	virtual void SetSwap(Bool_t s=kFALSE){swap=s;}
	virtual Bool_t GetSwap(void){return swap;}
	virtual Long_t GetEventNumber(void){return evtNum;}
	virtual Long_t GetNbOfBuffers(void){return nbuf;}
	virtual Short_t GetNbOfParameters(void){return mult;}
	virtual Short_t GetLabel(Int_t i){return label[i];}
	virtual Short_t GetValue(Int_t i){return valeur[i];}
 
    ClassDef(FNetEventReader,1)
 
}; 
 
#endif 
 
