//
// Patron pour la generation d'une classe derivee dans ROOT
// MReaderDeviceManager derive de TGCompositeFrame
//

#ifndef MReaderDeviceManager_h
#define MReaderDeviceManager_h

#include "TROOT.h"
#include "TGClient.h"
#include "FEventProcessor.h"
#include "FBlockDataReader.h"
#include "MCBLabel.h"
#include "MTELabel.h"

class MReaderDeviceManager : public TGCompositeFrame {

	enum ElementId
	{
	 CB_loDR,
	 CB_loDev,
	};

   protected:
	FEventProcessor		*fept;
	TList			*listOfDataReaders;
	TList			*listOfDevices;
	MCBLabel		*loDataReaders;
	MCBLabel		*loDevices;
	FBlockDataReader		*currentBlockDataReader;
	FDevice			*currentDevice;
	TGCompositeFrame	*deviceGUI;

   protected:
        virtual void Init(void);       // initialisation

   public:
	MReaderDeviceManager(const TGWindow *p=0,UInt_t w=500,UInt_t h=300);
#if ROOT_VERSION_CODE <= ROOT_VERSION(5,12,0)
	MReaderDeviceManager(const MReaderDeviceManager &ap); // constructeur par copie
#endif

	~MReaderDeviceManager(void);		// desctructeur;

#if ROOT_VERSION_CODE >= ROOT_VERSION(3,4,0)
   	virtual void Copy(TObject & obj) const;
#else
   	virtual void Copy(TObject & obj);
#endif
	MReaderDeviceManager& operator = (const MReaderDeviceManager &hp); // operateur =

	virtual Bool_t ProcessMessage(Long_t msg, Long_t par1, Long_t par2);

	virtual FEventProcessor *GetFEventProcessor(void)
		{return fept;}
	virtual void SetFEventProcessor(FEventProcessor *f=0)
		{fept=f;}

	virtual void BuildListOfDataReadersAndDevices();
	virtual void SelectDataReader(Char_t *sel);
	virtual void SelectDevice(Char_t *sel);
	virtual void SetListOfDevices(void);
	virtual void ShowDeviceGUI(void);

	virtual void ChangeTree(void);

	// Signals
	virtual void DataReaderChanged(void) {Emit("DataReaderChanged()");} // *SIGNAL*

	virtual void Reset(void);
	virtual void WarningBox(char const *title="Warning",char const *msg="Warning");

    ClassDef(MReaderDeviceManager,1)

};

#endif

