//
// Implementation de la classe DataMerger
//

#ifndef DataMerger_C
#define DataMerger_C

#include "DataMerger.h"
#include "FParamBrutSimple.h"
#include "FParamBrutVector.h"
#include "Riostream.h"

//ClassImp(DataMerger)

//____________________________________________________________________
DataMerger::DataMerger(void):TNamed()
{
//
// Constructeur
//
chanToNs=7.8125e-3;
tsMax=-1.;
}

//____________________________________________________________________
DataMerger::~DataMerger(void)
{
//
// Destructeur
//
}

//____________________________________________________________________
void DataMerger::AddTriggerParam(FParam *px, FParam *TSx, 
                                 FParam *&vOutTrig, FParam *&vOutTrigTS)
{
//
// Ajout d'un parametre Trigger et de son timeStamp
//
pTrigger.push_back(px);
pTriggerTS.push_back(TSx);
// On genere les parametres de sortie associes
if(!vOutTrig)
 {
 cout << "Creation du parametre associe a " << px->GetName() << endl;
 vOutTrig=new FParamBrutVector();
 vOutTrig->SetName(Form("%s_%s",this->GetName(),px->GetName()));
 vOutTrig->SetNbValuesMax(1);
 }
if(!vOutTrigTS)
 {
 cout << "Creation du parametre associe a " << TSx->GetName() << endl;
 vOutTrigTS=new FParamBrutVector();
 vOutTrigTS->SetName(Form("%s_%s",this->GetName(),TSx->GetName()));
 vOutTrigTS->SetNbValuesMax(1);
 }
vOutParam.push_back(vOutTrig);
vOutParam.push_back(vOutTrigTS);
valTS *v=new valTS();
vPtrig.push_back(v);
nt=pTrigger.size();
nbTrig.resize(nt);
}

//____________________________________________________________________
void DataMerger::AddSlaveParam(FParam *ps, FParam *TSs,
                               FParam *&vOut, FParam *&vOutTS)
{
//
// Ajout des parametres esclaves
//
pSlave.push_back(ps);
pSlaveTS.push_back(TSs);
// On genere les parametres de sortie associes
if(! vOut)
 {
 cout << "Creation du parametre associe a " << ps->GetName() << endl;
 vOut=new FParamBrutVector();
 vOut->SetName(Form("%s_%s",this->GetName(),ps->GetName()));
 vOut->SetNbValuesMax(1);
 }
if(! vOutTS)
 {
 cout << "Creation du parametre associe a " << TSs->GetName() << endl;
 vOutTS=new FParamBrutVector();
 vOutTS->SetName(Form("%s_%s",this->GetName(),TSs->GetName()));
 vOutTS->SetNbValuesMax(1);
 }
vOutParam.push_back(vOut);
vOutParam.push_back(vOutTS);
valTS *v=new valTS();
vPslave.push_back(v);
ns=pSlave.size();
}

//____________________________________________________________________
void DataMerger::FindTSmin(void)
{
 //
 // On va reperer le trigger qui a le TS minimum
 //
 tsMin=tsMax;
 itMin=-1;
 for(Int_t it=0;it<nt;it++)
  {
  Double_t ts=(*vPtrig[it])[0][1];   
  if(ts < tsMin)
   {
   tsMin=ts;itMin=it;
   }
  }
}

//____________________________________________________________________
Bool_t DataMerger::CheckTriggerVectors(void)
{
 //
 // On va verifier qu'il y a au moins une valeur dans chaque vecteur de valeur des trigger
 // et en meme temps on va reperer celui qui a le TS minimum
 //
 Bool_t ok=kTRUE;
 Int_t it=0;
 tsMin=tsMax;
 itMin=-1;
 while(ok && it < nt)
  {
  ok=(ok &&(vPtrig[it]->size() > 0));
  //cout << "   CTV " << it << " " << vPtrig[it]->size() << " " << ok << endl;
  it++;
  }
 
 return ok;
}

//____________________________________________________________________
void DataMerger::CountTriggersInWindow(void)
{
 // 
 // On va maintenant compter les triggers qui sont dans la fenetre de decision
 //
 //cout << "Entree dans CountTriggersInWindow() " << endl;
 for(Int_t it=0;it<nt;it++)
  {
  nbTrig[it]=0; // on initialise le nombre de valeurs a 0
  valTS *vec=vPtrig[it];
  Int_t nbt=(*vec).size();
  // On va maintenant scanner les valeurs presentes dans ce tableau
  Int_t i=0;
  while(i < nbt)
   {
   Double_t ts=(*vec)[i][1]-tsMin; // on recupere le TS relatif
   if(ts >= 0. && ts <= dGate) // s'il est dans la fenetre de decision
    {
    nbTrig[it]++; // on incremente le compteur
    }
   i++;
   }
  //cout << "   Count Trigger " << it << " : " << nbTrig[it] << endl;
  }
}

//____________________________________________________________________
Double_t DataMerger::ComputeTriggerTime(void)
{
 // 
 // On va calcuer le temps de reference du trigger: on va prendre le temps moyen de tous les triggers
 // dans la fenetre de decision
 //
 Double_t timeRef=0;
 Int_t nval=0;
 for(Int_t it=0;it<nt;it++)
  {
  valTS *vec=vPtrig[it];
  Int_t nbt=(*vec).size();
  // On va maintenant scanner les valeurs presentes dans ce tableau
  Int_t i=0;
  while(i < nbt)
   {
   Double_t ts=(*vec)[i][1]-tsMin; // on recupere le TS relatif
   if(ts >= 0. && ts <= dGate) // s'il est dans la fenetre de decision
    {
    nval++; // on incremente le compteur
    timeRef+=(*vec)[i][1];
    }
   i++;
   }
  }
 if(nval > 0) timeRef/=nval;
 return timeRef;
}

//____________________________________________________________________
Bool_t DataMerger::Decision(void)
{
 // 
 // On va prendre la decition sur le nombre de triggers dans la fenetre
 //
 Bool_t ok=kTRUE;
 for(Int_t it=0;it<nt;it++)
  {
  ok=(ok && nbTrig[it]==1);
  }
 return ok;
}

//____________________________________________________________________
Bool_t DataMerger::CheckTrigger(void)
{
 // On va verifier qu'il y a au moins une valeur dans chaque vecteur de valeur des trigger
 // et en meme temps on va reperer celui qui a le TS minimum
 // On sait maintenant qui a le plus petit TS. On va regarder si on peut prendre une decision, 
 // c'est a dire si tsMax est plus grand que la limite de decision tsMin+dGate
 //
 Bool_t ok = kTRUE;
 Bool_t loop = kTRUE;
 
 //cout << "Entree dans CheckTrigger()... " << endl;
 
 while(loop)
  {
  loop=kFALSE;
  ok=CheckTriggerVectors(); // on verifie que tous les triggers ont une valeur
  itMin=-1;
  if(ok) // si c'est le cas, on calcule tsMin
   {
   FindTSmin();
   ok=(tsMax > tsMin+dGate); // on verifie alors que tous les elements sont la
   //cout << "Condition TSmin : " << ok << " " << tsMin << " " << itMin << endl;
   }
 
 // 
 // On va maintenant compter les triggers qui sont dans la fenetre de decision
 //
  if(ok)
   {
   CountTriggersInWindow();
 //
 // A ce point on peut prendre la decision sur le nombre de triggers dans la fentre de decision
 //
   ok=Decision();
   //cout << "Decision : " << ok << endl;
 // Si ce n'est pas bon, il faut enlever la donnee correspondant a tsMin
   if(!ok && itMin >= 0)
    {
    valTS *vec=vPtrig[itMin];
    //cout << "On va eliminier trigger " << itMin << " : " << (*vec)[0][0] << " / " << (*vec)[0][1] << endl;
    vec->erase((*vec).begin());
 // et il faut recommencer la procedure avec le nouvel etat des tableaux des triggers
    loop=kTRUE;
    }
   }
  }
 //cout << "Sortie CheckTrigger() " << ok << endl; 
 return ok;
}

//____________________________________________________________________
void DataMerger::MergeData(void)
{
 // On va collecter les donnees qui sont dans la fenetre de collection et 
 // ajuster les valeurs des parametres presents dans vOutParam
 //
 //cout << "Entree dans MergeData() " << endl;
 Double_t timeRef=ComputeTriggerTime();
 //cout << "Time Ref : "  << timeRef << endl;
 Int_t iOut=0;
 //
 // On collecte les donnees des triggers
 //
 for(Int_t it=0;it<nt;it++)
  {
  valTS *vec=vPtrig[it];
  Int_t nbt=(*vec).size();
  // On va maintenant scanner les valeurs presentes dans ce tableau
  Int_t i=0;
  Double_t ts=-1.;
  Int_t nval=0;
  //cout << "Val trigger : " << it << endl;
  while(i < nbt && ts <= cGate)
   {
   ts=(*vec)[i][1]-(timeRef+cDelay); // on recupere le TS relatif
   //cout << "          " << it<< " , " << i << " : " << ts << endl;
   if(ts >= 0. && ts <= cGate) // s'il est dans la fenetre de decision
    {
    //cout << "          Ok : " <<vOutParam[iOut] << " / " << vOutParam[iOut+1] << endl;
    vOutParam[iOut]->SetValue((*vec)[i][0],nval);
    vOutParam[iOut+1]->SetValue((*vec)[i][1],nval);
    nval++;
    // On efface cette valeur du tableau
    //cout << "On efface l'element " << i << endl;
    vec->erase((*vec).begin()+i);
    nbt--;
    //cout << "fin ajout "<< endl; 
    }
   else if(ts < 0)
    {
    //cout << "          TS trop bas" << endl;
    vec->erase((*vec).begin()+i);
    nbt--;
    }
   else
    {
    i++;
    }
   }
  iOut+=2;
  }
 //
 // On collecte les donnees des esclaves
 //
 for(Int_t is=0;is<ns;is++)
  {
  valTS *vec=vPslave[is];
  Int_t nbs=(*vec).size();
  // On va maintenant scanner les valeurs presentes dans ce tableau
  Int_t i=0;
  Double_t ts=-1.;
  Int_t nval=0;
  //cout << "Val slave : " << is << endl;
  while(i < nbs && ts <= cGate)
   {
   ts=(*vec)[i][1]-(timeRef+cDelay); // on recupere le TS relatif
   //cout << "          " << is<< " , " << i << " : " << ts << endl;
   if(ts >= 0. && ts <= cGate) // s'il est dans la fenetre de decision
    {
    //cout << "          Ok " << endl;
    vOutParam[iOut]->SetValue((*vec)[i][0],nval);
    vOutParam[iOut+1]->SetValue((*vec)[i][1],nval);
    nval++;
    // On efface cette valeur du tableau
    vec->erase((*vec).begin()+i);
    nbs--;
    }
   else if(i==0 && ts < 0)
    {
    //cout << "          TS trop bas" << endl;
    vec->erase((*vec).begin()+i);
    nbs--;
    }
   else
    {
    i++;
    }
   }
  iOut+=2;
  }
 //cout << "Sortie de MergeData() " << endl;
}

//____________________________________________________________________
void DataMerger::Process(void)
{

 //cout << "Process de " << GetName() << endl;
 
 //
 // On ajoute une valeur a vPtrig si necessaire
 //
 for(Int_t it=0;it<nt;it++)
  {
  if(pTrigger[it]->IsOn() && pTriggerTS[it]->IsOn())
   {
   Double_t *val=new Double_t[2];
   val[0]=pTrigger[it]->GetValue();
   val[1]=pTriggerTS[it]->GetValue()*chanToNs;
   if(val[1] > tsMax) tsMax=val[1];
   vPtrig[it]->push_back(val);
   }
  }
 
 //
 // On ajoute les valeurs a vPslave si necessaire
 //
 for(Int_t is=0;is<ns;is++)
  {
  if(pSlave[is]->IsOn() && pSlaveTS[is]->IsOn())
   {
   Double_t *val=new Double_t[2];
   val[0]=pSlave[is]->GetValue();
   val[1]=pSlaveTS[is]->GetValue()*chanToNs;
   if(val[1] > tsMax) tsMax=val[1];
   vPslave[is]->push_back(val);
   }
  }
 
 //PrintValues();

 //
 // A ce point, on peut regarder si le trigger est bon
 //
 Bool_t ok=CheckTrigger();
 //
 // Si c'est bon, on va collecter les donnees et ajuster les valeurs des vOutParams
 //
 if(ok)
  {
  MergeData();
  }
 //cout << "Fin Process de " << GetName() << endl;
 //cout << "------------------------------------------------------" << endl;
} 

//____________________________________________________________________
void DataMerger::Reset(void)
{
//
// Initialise les vecteurs
//

} 

//____________________________________________________________________
void DataMerger::ls(Option_t *option) const
{
//
// Affiche le contenu du DataMerger
//
cout << "=============================================================" << endl;
cout << GetName() << "(" << ClassName() << ") : " << endl;
cout << "Triggers : " << pTrigger.size() << endl;
for(Int_t i=0;i<pTrigger.size();i++)
 {
 cout << i+1 << " --> " <<pTrigger[i]->GetName() << " / " << 
 pTriggerTS[i]->GetName() << endl;
 }
cout << "-------------------------------------------------------------" << endl;
cout << "Esclaves : " << pSlave.size() << endl;
for(Int_t i=0;i<pSlave.size();i++)
 {
 cout << i+1 << " --> " << pSlave[i]->GetName() << " / " <<
 pSlaveTS[i]->GetName() << endl;
 }
cout << "-------------------------------------------------------------" << endl;
cout << "Parametres de sortie : " << endl;
for(Int_t i=0;i<vOutParam.size();i+=2)
 {
 cout << i+1 << " --> " << vOutParam[i]->GetName() << " / " <<
 vOutParam[i+1]->GetName() << endl;
 }
cout << "=============================================================" << endl;

}

//____________________________________________________________________
void DataMerger::PrintValues(void)
{
//
// Affiche le contenu des vecteurs de donnees
//
 cout.precision(25);
 for(Int_t it=0;it<nt;it++)
  {
  cout << "Trigger " << it << " (" <<pTrigger[it]->GetName() << ") : " << (*vPtrig[it]).size() << " valeurs." << endl;
  for(Int_t iv=0;iv<(*vPtrig[it]).size();iv++)
   {
   cout << "     " << iv << " : " << (*vPtrig[it])[iv][0] << " / " << (*vPtrig[it])[iv][1];
   if(iv>0) cout << ", Delta Prec : " << (*vPtrig[it])[iv][1]-(*vPtrig[it])[iv-1][1];
   if(itMin>=0) cout << ", Delta Tmin : " << (*vPtrig[it])[iv][1]-tsMin;
   cout << endl;
   }
  }
 
 for(Int_t is=0;is<ns;is++)
  {
  cout << "Slave " << is << " (" <<pSlave[is]->GetName()<< ") : " << (*vPslave[is]).size() << " valeurs." << endl;
  for(Int_t iv=0;iv<(*vPslave[is]).size();iv++)
   {
   cout << "     " << iv << " : " << (*vPslave[is])[iv][0] << " / " << (*vPslave[is])[iv][1];
   if(iv>0) cout << ", Delta Prec : " << (*vPslave[is])[iv][1]-(*vPslave[is])[iv-1][1];
   if(itMin>=0) cout << ", Delta Tmin : " << (*vPslave[is])[iv][1]-tsMin;
   cout << endl;
   }
  }
 
 cout << "TS max " << tsMax << endl;
 cout << "TS min " << tsMin << " / " << itMin;
 if(itMin > 0) cout << ", Delta : " << tsMax-tsMin;
 cout << endl;

}

#endif
